==> jei-1.20.1-fabric-15.20.0.116-sources.jar: mezz/jei/gui/recipes/RecipeSortUtil.java <==
package mezz.jei.gui.recipes;

import mezz.jei.api.gui.IRecipeLayoutDrawable;
import mezz.jei.api.gui.ingredient.IRecipeSlotView;
import mezz.jei.api.recipe.RecipeIngredientRole;
import mezz.jei.api.recipe.category.IRecipeCategory;
import mezz.jei.api.recipe.transfer.IRecipeTransferManager;
import net.minecraft.class_1703;
import net.minecraft.class_310;
import net.minecraft.class_746;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;

public class RecipeSortUtil {
	private static final Comparator<RecipeLayoutWithButtons<?>> CRAFTABLE_COMPARATOR = createCraftableComparator();

	public static List<IRecipeCategory<?>> sortRecipeCategories(
		List<IRecipeCategory<?>> recipeCategories,
		IRecipeTransferManager recipeTransferManager
	) {
		class_310 minecraft = class_310.method_1551();
		class_746 player = minecraft.field_1724;
		if (player == null) {
			return recipeCategories;
		}
		class_1703 openContainer = player.field_7512;
		//noinspection ConstantConditions
		if (openContainer == null) {
			return recipeCategories;
		}

		Comparator<IRecipeCategory<?>> comparator = Comparator.comparing((IRecipeCategory<?> r) -> {
			var recipeTransferHandler = recipeTransferManager.getRecipeTransferHandler(openContainer, r);
			return recipeTransferHandler.isPresent();
		})
			.reversed();

		return recipeCategories.stream()
			.sorted(comparator)
			.toList();
	}

	public static Comparator<RecipeLayoutWithButtons<?>> getCraftableComparator() {
		return CRAFTABLE_COMPARATOR;
	}

	private static Comparator<RecipeLayoutWithButtons<?>> createCraftableComparator() {
		return Comparator.comparingInt(r -> {
			IRecipeLayoutDrawable<?> recipeLayout = r.recipeLayout();
			List<IRecipeSlotView> inputSlotViews = recipeLayout.getRecipeSlotsView()
				.getSlotViews(RecipeIngredientRole.INPUT);

			RecipeTransferButton transferButton = r.transferButton();
			int missingCount = transferButton.getMissingCountHint();
			if (missingCount == -1) {
				return 0;
			}

			int ingredientCount = ingredientCount(inputSlotViews);
			if (ingredientCount == 0) {
				return 0;
			}

			int matchCount = ingredientCount - missingCount;
			int matchPercent = 100 * matchCount / ingredientCount;
			return -matchPercent;
		});
	}

	private static int ingredientCount(List<IRecipeSlotView> inputSlotViews) {
		int count = 0;
		for (IRecipeSlotView i : inputSlotViews) {
			if (!i.isEmpty()) {
				count++;
			}
		}
		return count;
	}
}

==> jei-1.21.1-fabric-19.25.1.334-sources.jar: mezz/jei/gui/recipes/RecipeSortUtil.java <==
package mezz.jei.gui.recipes;

import mezz.jei.api.gui.IRecipeLayoutDrawable;
import mezz.jei.api.gui.ingredient.IRecipeSlotView;
import mezz.jei.api.gui.ingredient.IRecipeSlotsView;
import mezz.jei.api.recipe.RecipeIngredientRole;
import mezz.jei.api.recipe.category.IRecipeCategory;
import mezz.jei.api.recipe.transfer.IRecipeTransferManager;
import net.minecraft.class_1703;
import net.minecraft.class_310;
import net.minecraft.class_746;
import java.util.Comparator;
import java.util.List;

public class RecipeSortUtil {
	private static final Comparator<IRecipeLayoutWithButtons<?>> COMPARATOR = createComparator();

	public static List<IRecipeCategory<?>> sortRecipeCategories(
		List<IRecipeCategory<?>> recipeCategories,
		IRecipeTransferManager recipeTransferManager
	) {
		class_310 minecraft = class_310.method_1551();
		class_746 player = minecraft.field_1724;
		if (player == null) {
			return recipeCategories;
		}
		class_1703 openContainer = player.field_7512;
		//noinspection ConstantConditions
		if (openContainer == null) {
			return recipeCategories;
		}

		Comparator<IRecipeCategory<?>> comparator = Comparator.comparing((IRecipeCategory<?> r) -> {
			var recipeTransferHandler = recipeTransferManager.getRecipeTransferHandler(openContainer, r);
			return recipeTransferHandler.isPresent();
		})
			.reversed();

		return recipeCategories.stream()
			.sorted(comparator)
			.toList();
	}

	public static Comparator<IRecipeLayoutWithButtons<?>> getComparator() {
		return COMPARATOR;
	}

	private static Comparator<IRecipeLayoutWithButtons<?>> createComparator() {
		return Comparator.comparingInt(r -> {
			IRecipeLayoutDrawable<?> recipeLayout = r.getRecipeLayout();

			int missingCount = r.getMissingCountHint();
			if (missingCount == -1) {
				return 0;
			}

			IRecipeSlotsView recipeSlotsView = recipeLayout.getRecipeSlotsView();
			int ingredientCount = inputCount(recipeSlotsView);
			if (ingredientCount == 0) {
				return 0;
			}

			int matchCount = ingredientCount - missingCount;
			int matchPercent = 100 * matchCount / ingredientCount;
			return -matchPercent;
		});
	}

	private static int inputCount(IRecipeSlotsView recipeSlotsView) {
		int count = 0;
		for (IRecipeSlotView i : recipeSlotsView.getSlotViews()) {
			if (i.getRole() == RecipeIngredientRole.INPUT && !i.isEmpty()) {
				count++;
			}
		}
		return count;
	}
}
